@@ -1,8 +1,8 @@
 package Test::Instruction;
 
-use 5.006; use strict; use warnings; our $VERSION = '0.01';
+use 5.006; use strict; use warnings; our $VERSION = '0.02';
 use Compiled::Params::OO qw/cpo/;
-use Types::Standard qw/Optional Str Int Bool Any CodeRef/;
+use Types::Standard qw/Optional Str Int Bool Any CodeRef ArrayRef HashRef/;
 use B qw/svref_2object/;
 use Switch::Again qw/switch/;
 use Test::More;
@@ -17,16 +17,28 @@ BEGIN {
 	);
 	$validate = cpo(
 		instruction => {
-		    instance  => Optional->of(Any),
-		    meth      => Optional->of(Str),
-		    func      => Optional->of(CodeRef),
-		    args      => Optional->of(Any),
-		    args_list => Optional->of(Bool),
-		    test      => Optional->of(Str),
-		    expected  => Optional->of(Any),
-		    catch     => Optional->of(Bool),
-		    key       => Optional->of(Str),
-		    index     => Optional->of(Int),
+			instance  => Optional->of(Any),
+			meth      => Optional->of(Str),
+			func      => Optional->of(CodeRef),
+			args      => Optional->of(Any),
+			args_list => Optional->of(Bool),
+			test      => Optional->of(Str),
+			expected  => Optional->of(Any),
+			catch     => Optional->of(Bool),
+			key       => Optional->of(Str),
+			index     => Optional->of(Int),
+		},
+		instructions => {
+			name => Str,
+			run => ArrayRef,
+			build => Optional->of(HashRef),
+			instance => Optional->of(Any)
+		},
+		build => {
+			class => Str,
+			new => Optional->of(Str),
+			args => Optional->of(Any),
+			args_list => Optional->of(Bool)
 		}
 	);
 }
@@ -301,16 +313,77 @@ sub instruction {
 		};
 }
 
-sub instructions { ... }
+sub instructions { 
+	my $instructions = $validate->instructions->(@_);
+
+	ok(1, sprintf "instructions: %s", $instructions->name);
+
+	my $instance = $instructions->build ? _build($instructions->build) : $instructions->instance;
+
+	my %test_info = (
+		fail => 0,
+		tested => 0,
+	);
+
+	for my $instruction (@{$instructions->run}) {
+		$test_info{tested}++;
+		if (my $subtests = delete $instruction->{instructions}) {
+			my ($test_name, $new_instance) = _run_the_code(
+				$validate->instruction->(
+					instance => $instance,
+					%{$instruction}
+				)
+			);
+			
+			$test_info{fail}++
+				unless instruction(
+					instance => $new_instance,
+					test => $instruction->{test},
+					expected => $instruction->{expected}
+				);
+
+			instructions(
+				instance => $new_instance,
+				run => $subtests,
+				name => sprintf "Subtest -> %s -> %s", $instructions->name, $test_name
+			);
+			next;
+		}
+
+		$test_info{fail}++
+			unless instruction(
+				instance => $instance,
+				%{$instruction}
+			);
+	}
+	
+	$test_info{ok} = $test_info{fail} ? 0 : 1;
+	return ok(
+		$test_info{ok},
+		sprintf(
+			"instructions: %s - tested %d instructions - success: %d - failure: %d",
+			$instructions->name,
+			$test_info{tested},
+			($test_info{tested} - $test_info{fail}),
+			$test_info{fail}
+		)
+	);
+}
 
 sub finish {
 	my $done_testing = done_testing(shift);
 	return $done_testing;
 }
 
+
+sub _build {
+	my $build = $validate->build->(@_);
+	my $new = $build->new || 'new';
+	return $build->class->$new($build->args_list ? @{ $build->args } : $build->args);
+}
+
 sub _run_the_code {
 	my $instruction = shift;
-
 	if ($instruction->meth) {
 		my $meth = $instruction->meth;
 		return (
@@ -346,7 +419,7 @@ Test::Instruction - A test framework
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
@@ -377,6 +450,45 @@ Version 0.01
 
 	finish(3);
 
+	...
+
+	use Test::Instruction qw/all/;
+
+	instructions(
+		name => 'Checking Many Things',
+		build => {
+			class => 'Foo',
+		},
+		run => [
+			{
+				test => 'hash',
+				expected => {}
+			},
+			{
+				test => 'true',
+				meth => 'true',
+			},
+			{
+				test => 'false',
+				meth => 'false',
+			},
+			{
+				test => 'ok',
+				meth => "chain",
+				instructions => [
+					{
+						test => 'hash',
+						expected => {
+							a => "b"
+						}
+					}
+				]
+			}
+		],
+	);
+
+	finish();
+
 =head1 EXPORT
 
 =head2 instruction
@@ -495,6 +607,8 @@ key - required when testing - ref_key_* and list_key_*
  
 =cut
 
+=head2 instructions
+
 =cut
 
 =head2 finish
